@@ -1,38 +1,35 @@
-// ==========================================================================
-// This software is subject to the provisions of the Zope Public License,
-// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
-// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
-// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-// FOR A PARTICULAR PURPOSE.
-// ==========================================================================
-
 using System;
-using System.IO;
 using System.Collections;
-using System.Collections.Specialized;
+using System.IO;
+using System.Collections.Concurrent;
 using System.Collections.Generic;
+using System.Diagnostics;
 using System.Reflection;
-using System.Reflection.Emit;
-
-namespace Python.Runtime {
+using System.Threading;
 
+namespace Python.Runtime
+{
     /// <summary>
-    /// The AssemblyManager maintains information about loaded assemblies  
+    /// The AssemblyManager maintains information about loaded assemblies
     /// namespaces and provides an interface for name-based type lookup.
     /// </summary>
-
-    internal class AssemblyManager {
-
-        static Dictionary<string, Dictionary<Assembly, string>> namespaces;
+    internal class AssemblyManager
+    {
+        // modified from event handlers below, potentially triggered from different .NET threads
+        // therefore this should be a ConcurrentDictionary
+        static ConcurrentDictionary<string, ConcurrentDictionary<Assembly, string>> namespaces;
         //static Dictionary<string, Dictionary<string, string>> generics;
         static AssemblyLoadEventHandler lhandler;
         static ResolveEventHandler rhandler;
+        // updated only under GIL?
         static Dictionary<string, int> probed;
-        static List<Assembly> assemblies;
+        // modified from event handlers below, potentially triggered from different .NET threads
+        static AssemblyList assemblies;
         internal static List<string> pypath;
 
-        private AssemblyManager() {}
+        private AssemblyManager()
+        {
+        }
 
         //===================================================================
         // Initialization performed on startup of the Python runtime. Here we
@@ -40,12 +37,13 @@ namespace Python.Runtime {
         // names, and register to be notified of new assembly loads.
         //===================================================================
 
-        internal static void Initialize() {
-            namespaces = new 
-                         Dictionary<string, Dictionary<Assembly, string>>(32);
+        internal static void Initialize()
+        {
+            namespaces = new
+                ConcurrentDictionary<string, ConcurrentDictionary<Assembly, string>>();
             probed = new Dictionary<string, int>(32);
             //generics = new Dictionary<string, Dictionary<string, string>>();
-            assemblies = new List<Assembly>(16);
+            assemblies = new AssemblyList(16);
             pypath = new List<string>(16);
 
             AppDomain domain = AppDomain.CurrentDomain;
@@ -53,14 +51,21 @@ namespace Python.Runtime {
             lhandler = new AssemblyLoadEventHandler(AssemblyLoadHandler);
             domain.AssemblyLoad += lhandler;
 
-            rhandler = new ResolveEventHandler(ResolveHandler);        
+            rhandler = new ResolveEventHandler(ResolveHandler);
             domain.AssemblyResolve += rhandler;
 
             Assembly[] items = domain.GetAssemblies();
-            for (int i = 0; i < items.Length; i++) {
-                Assembly a = items[i];
-                assemblies.Add(a);
-                ScanAssembly(a);
+            foreach (var a in items)
+            {
+                try
+                {
+                    ScanAssembly(a);
+                    assemblies.Add(a);
+                }
+                catch (Exception ex)
+                {
+                    Debug.WriteLine(string.Format("Error scanning assembly {0}. {1}", a, ex));
+                }
             }
         }
 
@@ -69,7 +74,8 @@ namespace Python.Runtime {
         // Cleanup resources upon shutdown of the Python runtime.
         //===================================================================
 
-        internal static void Shutdown() {
+        internal static void Shutdown()
+        {
             AppDomain domain = AppDomain.CurrentDomain;
             domain.AssemblyLoad -= lhandler;
             domain.AssemblyResolve -= rhandler;
@@ -77,14 +83,15 @@ namespace Python.Runtime {
 
 
         //===================================================================
-        // Event handler for assembly load events. At the time the Python 
+        // Event handler for assembly load events. At the time the Python
         // runtime loads, we scan the app domain to map the assemblies that
         // are loaded at the time. We also have to register this event handler
-        // so that we can know about assemblies that get loaded after the 
+        // so that we can know about assemblies that get loaded after the
         // Python runtime is initialized.
         //===================================================================
 
-        static void AssemblyLoadHandler(Object ob, AssemblyLoadEventArgs args){
+        static void AssemblyLoadHandler(Object ob, AssemblyLoadEventArgs args)
+        {
             Assembly assembly = args.LoadedAssembly;
             assemblies.Add(assembly);
             ScanAssembly(assembly);
@@ -99,12 +106,14 @@ namespace Python.Runtime {
         // we loaded from Python which also needs to be found on PYTHONPATH.
         //===================================================================
 
-        static Assembly ResolveHandler(Object ob, ResolveEventArgs args){
+        static Assembly ResolveHandler(Object ob, ResolveEventArgs args)
+        {
             string name = args.Name.ToLower();
-            for (int i = 0; i < assemblies.Count; i++) {
-                Assembly a = (Assembly)assemblies[i];
+            foreach (Assembly a in assemblies)
+            {
                 string full = a.FullName.ToLower();
-                if (full.StartsWith(name)) {
+                if (full.StartsWith(name))
+                {
                     return a;
                 }
             }
@@ -114,27 +123,31 @@ namespace Python.Runtime {
 
         //===================================================================
         // We __really__ want to avoid using Python objects or APIs when
-        // probing for assemblies to load, since our ResolveHandler may be 
+        // probing for assemblies to load, since our ResolveHandler may be
         // called in contexts where we don't have the Python GIL and can't
         // even safely try to get it without risking a deadlock ;(
         //
-        // To work around that, we update a managed copy of sys.path (which 
+        // To work around that, we update a managed copy of sys.path (which
         // is the main thing we care about) when UpdatePath is called. The
         // import hook calls this whenever it knows its about to use the
         // assembly manager, which lets us keep up with changes to sys.path
         // in a relatively lightweight and low-overhead way.
         //===================================================================
 
-        internal static void UpdatePath() {
+        internal static void UpdatePath()
+        {
             IntPtr list = Runtime.PySys_GetObject("path");
             int count = Runtime.PyList_Size(list);
-            if (count != pypath.Count) {
+            if (count != pypath.Count)
+            {
                 pypath.Clear();
                 probed.Clear();
-                for (int i = 0; i < count; i++) {
+                for (int i = 0; i < count; i++)
+                {
                     IntPtr item = Runtime.PyList_GetItem(list, i);
                     string path = Runtime.GetManagedString(item);
-                    if (path != null) {
+                    if (path != null)
+                    {
                         pypath.Add(path);
                     }
                 }
@@ -148,26 +161,32 @@ namespace Python.Runtime {
         // using standard load semantics (app base directory then GAC, etc.)
         //===================================================================
 
-        public static string FindAssembly(string name) {
+        public static string FindAssembly(string name)
+        {
             char sep = Path.DirectorySeparatorChar;
             string path;
             string temp;
 
-            for (int i = 0; i < pypath.Count; i++) {
+            for (int i = 0; i < pypath.Count; i++)
+            {
                 string head = pypath[i];
-                if (head == null || head.Length == 0) {
+                if (head == null || head.Length == 0)
+                {
                     path = name;
                 }
-                else {
+                else
+                {
                     path = head + sep + name;
                 }
 
                 temp = path + ".dll";
-                if (File.Exists(temp)) {
+                if (File.Exists(temp))
+                {
                     return temp;
                 }
                 temp = path + ".exe";
-                if (File.Exists(temp)) {
+                if (File.Exists(temp))
+                {
                     return temp;
                 }
             }
@@ -180,12 +199,15 @@ namespace Python.Runtime {
         // given a simple assembly name. Returns the assembly if loaded.
         //===================================================================
 
-        public static Assembly LoadAssembly(string name) {
+        public static Assembly LoadAssembly(string name)
+        {
             Assembly assembly = null;
-            try {
+            try
+            {
                 assembly = Assembly.Load(name);
             }
-            catch (System.Exception e) {
+            catch (System.Exception e)
+            {
                 //if (!(e is System.IO.FileNotFoundException)) {
                 //    throw;
                 //}
@@ -198,12 +220,45 @@ namespace Python.Runtime {
         // Loads an assembly using an augmented search path (the python path).
         //===================================================================
 
-        public static Assembly LoadAssemblyPath(string name) {
+        public static Assembly LoadAssemblyPath(string name)
+        {
             string path = FindAssembly(name);
             Assembly assembly = null;
-            if (path != null) {
-                try   { assembly = Assembly.LoadFrom(path); }
-                catch {}
+            if (path != null)
+            {
+                try
+                {
+                    assembly = Assembly.LoadFrom(path);
+                }
+                catch
+                {
+                }
+            }
+            return assembly;
+        }
+
+        /// <summary>
+        /// Loads an assembly using full path.
+        /// </summary>
+        /// <param name="name"></param>
+        /// <returns></returns>
+        public static Assembly LoadAssemblyFullPath(string name)
+        {
+            Assembly assembly = null;
+            if (Path.IsPathRooted(name))
+            {
+                if (!Path.HasExtension(name))
+                    name = name + ".dll";
+                if (File.Exists(name))
+                {
+                    try
+                    {
+                        assembly = Assembly.LoadFrom(name);
+                    }
+                    catch
+                    {
+                    }
+                }
             }
             return assembly;
         }
@@ -212,10 +267,12 @@ namespace Python.Runtime {
         // Returns an assembly that's already been loaded
         //===================================================================
 
-        public static Assembly FindLoadedAssembly(string name) {
-            for (int i = 0; i < assemblies.Count; i++) {
-                Assembly a = (Assembly)assemblies[i];
-                if (a.GetName().Name == name) {
+        public static Assembly FindLoadedAssembly(string name)
+        {
+            foreach (Assembly a in assemblies)
+            {
+                if (a.GetName().Name == name)
+                {
                     return a;
                 }
             }
@@ -223,7 +280,7 @@ namespace Python.Runtime {
         }
 
         //===================================================================
-        // Given a qualified name of the form A.B.C.D, attempt to load 
+        // Given a qualified name of the form A.B.C.D, attempt to load
         // an assembly named after each of A.B.C.D, A.B.C, A.B, A. This
         // will only actually probe for the assembly once for each unique
         // namespace. Returns true if any assemblies were loaded.
@@ -234,26 +291,33 @@ namespace Python.Runtime {
         // Call ONLY for namespaces that HAVE NOT been cached yet.
         //===================================================================
 
-        public static bool LoadImplicit(string name, bool warn=true) {
+        public static bool LoadImplicit(string name, bool warn = true)
+        {
             string[] names = name.Split('.');
             bool loaded = false;
             string s = "";
             Assembly lastAssembly = null;
-            HashSet<Assembly> assemblies = null;
-            for (int i = 0; i < names.Length; i++) {
+            HashSet<Assembly> assembliesSet = null;
+            for (int i = 0; i < names.Length; i++)
+            {
                 s = (i == 0) ? names[0] : s + "." + names[i];
-                if (!probed.ContainsKey(s)) {
-                    if (assemblies == null) {
-                        assemblies = new HashSet<Assembly>(AppDomain.CurrentDomain.GetAssemblies());
+                if (!probed.ContainsKey(s))
+                {
+                    if (assembliesSet == null)
+                    {
+                        assembliesSet = new HashSet<Assembly>(AppDomain.CurrentDomain.GetAssemblies());
                     }
                     Assembly a = FindLoadedAssembly(s);
-                    if (a == null) {
+                    if (a == null)
+                    {
                         a = LoadAssemblyPath(s);
                     }
-                    if (a == null) {
+                    if (a == null)
+                    {
                         a = LoadAssembly(s);
                     }
-                    if (a != null && !assemblies.Contains(a)) {
+                    if (a != null && !assembliesSet.Contains(a))
+                    {
                         loaded = true;
                         lastAssembly = a;
                     }
@@ -265,9 +329,9 @@ namespace Python.Runtime {
             if (warn && loaded)
             {
                 string deprWarning = String.Format(
-                                "\nThe module was found, but not in a referenced namespace.\n" +
-                                "Implicit loading is deprecated. Please use clr.AddReference(\"{0}\").",
-                                    Path.GetFileNameWithoutExtension(lastAssembly.Location));
+                    "\nThe module was found, but not in a referenced namespace.\n" +
+                    "Implicit loading is deprecated. Please use clr.AddReference(\"{0}\").",
+                    Path.GetFileNameWithoutExtension(lastAssembly.Location));
                 Exceptions.deprecation(deprWarning);
             }
 
@@ -278,38 +342,39 @@ namespace Python.Runtime {
         //===================================================================
         // Scans an assembly for exported namespaces, adding them to the
         // mapping of valid namespaces. Note that for a given namespace
-        // a.b.c.d, each of a, a.b, a.b.c and a.b.c.d are considered to 
+        // a.b.c.d, each of a, a.b, a.b.c and a.b.c.d are considered to
         // be valid namespaces (to better match Python import semantics).
         //===================================================================
 
-        static void ScanAssembly(Assembly assembly) {
-
+        internal static void ScanAssembly(Assembly assembly)
+        {
             // A couple of things we want to do here: first, we want to
             // gather a list of all of the namespaces contributed to by
             // the assembly.
 
             Type[] types = assembly.GetTypes();
-            for (int i = 0; i < types.Length; i++) {
+            for (int i = 0; i < types.Length; i++)
+            {
                 Type t = types[i];
-                string ns = t.Namespace;
-                if ((ns != null) && (!namespaces.ContainsKey(ns))) {
+                string ns = t.Namespace ?? "";
+                if (!namespaces.ContainsKey(ns))
+                {
                     string[] names = ns.Split('.');
                     string s = "";
-                    for (int n = 0; n < names.Length; n++) {
+                    for (int n = 0; n < names.Length; n++)
+                    {
                         s = (n == 0) ? names[0] : s + "." + names[n];
-                        if (!namespaces.ContainsKey(s)) {
-                            namespaces.Add(s,
-                                           new Dictionary<Assembly, string>()
-                                           );
-                        }
+                        namespaces.TryAdd(s, new ConcurrentDictionary<Assembly, string>());
                     }
                 }
 
-                if (ns != null && !namespaces[ns].ContainsKey(assembly)) {
-                    namespaces[ns].Add(assembly, String.Empty);
+                if (ns != null)
+                {
+                    namespaces[ns].TryAdd(assembly, String.Empty);
                 }
 
-                if (ns != null && t.IsGenericTypeDefinition) {
+                if (ns != null && t.IsGenericTypeDefinition)
+                {
                     GenericUtil.Register(t);
                 }
             }
@@ -317,14 +382,12 @@ namespace Python.Runtime {
 
         public static AssemblyName[] ListAssemblies()
         {
-            AssemblyName[] names = new AssemblyName[assemblies.Count];
-            Assembly assembly;
-            for (int i=0; i < assemblies.Count; i++)
+            List<AssemblyName> names = new List<AssemblyName>(assemblies.Count);
+            foreach (Assembly assembly in assemblies)
             {
-                assembly = assemblies[i];
-                names.SetValue(assembly.GetName(), i);
+                names.Add(assembly.GetName());
             }
-            return names;
+            return names.ToArray();
         }
 
         //===================================================================
@@ -332,43 +395,65 @@ namespace Python.Runtime {
         // exported by an assembly loaded in the current app domain.
         //===================================================================
 
-        public static bool IsValidNamespace(string name) {
-            return namespaces.ContainsKey(name);
+        public static bool IsValidNamespace(string name)
+        {
+            return !String.IsNullOrEmpty(name) && namespaces.ContainsKey(name);
         }
 
+        //===================================================================
+        // Returns list of assemblies that declare types in a given namespace
+        //===================================================================
+
+        public static IEnumerable<Assembly> GetAssemblies(string nsname)
+        {
+            if (!namespaces.ContainsKey(nsname))
+                return new List<Assembly>();
+
+            return namespaces[nsname].Keys;
+        }
 
         //===================================================================
         // Returns the current list of valid names for the input namespace.
         //===================================================================
 
-        public static List<string> GetNames(string nsname) {
+        public static List<string> GetNames(string nsname)
+        {
             //Dictionary<string, int> seen = new Dictionary<string, int>();
             List<string> names = new List<string>(8);
 
             List<string> g = GenericUtil.GetGenericBaseNames(nsname);
-            if (g != null) {
-                foreach (string n in g) {
+            if (g != null)
+            {
+                foreach (string n in g)
+                {
                     names.Add(n);
                 }
             }
 
-            if (namespaces.ContainsKey(nsname)) {
-                foreach (Assembly a in namespaces[nsname].Keys) {
+            if (namespaces.ContainsKey(nsname))
+            {
+                foreach (Assembly a in namespaces[nsname].Keys)
+                {
                     Type[] types = a.GetTypes();
-                    for (int i = 0; i < types.Length; i++) {
+                    for (int i = 0; i < types.Length; i++)
+                    {
                         Type t = types[i];
-                        if (t.Namespace == nsname) {
+                        if ((t.Namespace ?? "") == nsname)
+                        {
                             names.Add(t.Name);
                         }
                     }
                 }
                 int nslen = nsname.Length;
-                foreach (string key in namespaces.Keys) {
-                    if (key.Length > nslen && key.StartsWith(nsname)) {
+                foreach (string key in namespaces.Keys)
+                {
+                    if (key.Length > nslen && key.StartsWith(nsname))
+                    {
                         //string tail = key.Substring(nslen);
-                        if (key.IndexOf('.') == -1) {
+                        if (key.IndexOf('.') == -1)
+                        {
                             names.Add(key);
-                        } 
+                        }
                     }
                 }
             }
@@ -381,18 +466,104 @@ namespace Python.Runtime {
         // type. Returns null if the named type cannot be found.
         //===================================================================
 
-        public static Type LookupType(string qname) {
-            for (int i = 0; i < assemblies.Count; i++) {
-                Assembly assembly = (Assembly)assemblies[i];
+        public static Type LookupType(string qname)
+        {
+            foreach (Assembly assembly in assemblies)
+            {
                 Type type = assembly.GetType(qname);
-                if (type != null) {
+                if (type != null)
+                {
                     return type;
                 }
             }
             return null;
         }
 
+        /// <summary>
+        /// Wrapper around List<Assembly> for thread safe access
+        /// </summary>
+        private class AssemblyList : IEnumerable<Assembly>{
+            private readonly List<Assembly> _list;
+            private readonly ReaderWriterLockSlim _lock;
+
+            public AssemblyList(int capacity) {
+                _list = new List<Assembly>(capacity);
+                _lock = new ReaderWriterLockSlim();
+            }
+
+            public int Count
+            {
+                get
+                {
+                    _lock.EnterReadLock();
+                    try {
+                        return _list.Count;
+                    }
+                    finally {
+                        _lock.ExitReadLock();
+                    }
+                }
+            }
+
+            public void Add(Assembly assembly) {
+                _lock.EnterWriteLock();
+                try
+                {
+                    _list.Add(assembly);
+                }
+                finally
+                {
+                    _lock.ExitWriteLock();
+                }
+            }
+
+            public IEnumerator GetEnumerator()
+            {
+                return ((IEnumerable<Assembly>) this).GetEnumerator();
+            }
+
+            /// <summary>
+            /// Enumerator wrapping around <see cref="AssemblyList._list"/>'s enumerator.
+            /// Acquires and releases a read lock on <see cref="AssemblyList._lock"/> during enumeration
+            /// </summary>
+            private class Enumerator : IEnumerator<Assembly>
+            {
+                private readonly AssemblyList _assemblyList;
+
+                private readonly IEnumerator<Assembly> _listEnumerator;
+
+                public Enumerator(AssemblyList assemblyList)
+                {
+                    _assemblyList = assemblyList;
+                    _assemblyList._lock.EnterReadLock();
+                    _listEnumerator = _assemblyList._list.GetEnumerator();
+                }
+
+                public void Dispose()
+                {
+                    _listEnumerator.Dispose();
+                    _assemblyList._lock.ExitReadLock();
+                }
+
+                public bool MoveNext()
+                {
+                    return _listEnumerator.MoveNext();
+                }
+
+                public void Reset()
+                {
+                    _listEnumerator.Reset();
+                }
+
+                public Assembly Current { get { return _listEnumerator.Current; } }
+
+                object IEnumerator.Current { get { return Current; } }
+            }
+
+            IEnumerator<Assembly> IEnumerable<Assembly>.GetEnumerator()
+            {
+                return new Enumerator(this);
+            }
+        }
     }
-
-
 }
