@@ -1,32 +1,22 @@
-// ==========================================================================
-// This software is subject to the provisions of the Zope Public License,
-// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
-// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
-// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-// FOR A PARTICULAR PURPOSE.
-// ==========================================================================
-
 using System;
 using System.Runtime.InteropServices;
 using System.Collections;
 using System.Reflection;
 
-namespace Python.Runtime {
-
+namespace Python.Runtime
+{
     /// <summary>
     /// Base class for extensions whose instances *share* a single Python
-    /// type object, such as the types that represent CLR methods, fields, 
+    /// type object, such as the types that represent CLR methods, fields,
     /// etc. Instances implemented by this class do not support subtyping.
     /// </summary>
-
-    internal abstract class ExtensionType : ManagedType {
-
-        public ExtensionType() : base() {
-
+    internal abstract class ExtensionType : ManagedType
+    {
+        public ExtensionType() : base()
+        {
             // Create a new PyObject whose type is a generated type that is
             // implemented by the particuar concrete ExtensionType subclass.
-            // The Python instance object is related to an instance of a 
+            // The Python instance object is related to an instance of a
             // particular concrete subclass with a hidden CLR gchandle.
 
             IntPtr tp = TypeManager.GetTypeHandle(this.GetType());
@@ -40,7 +30,7 @@ namespace Python.Runtime {
             IntPtr py = Runtime.PyType_GenericAlloc(tp, 0);
 
             GCHandle gc = GCHandle.Alloc(this);
-            Marshal.WriteIntPtr(py, ObjectOffset.magic(), (IntPtr)gc);
+            Marshal.WriteIntPtr(py, ObjectOffset.magic(tp), (IntPtr)gc);
 
             // We have to support gc because the type machinery makes it very
             // hard not to - but we really don't have a need for it in most
@@ -59,9 +49,10 @@ namespace Python.Runtime {
         // Common finalization code to support custom tp_deallocs.
         //====================================================================
 
-        public static void FinalizeObject(ManagedType self) {
+        public static void FinalizeObject(ManagedType self)
+        {
             Runtime.PyObject_GC_Del(self.pyHandle);
-            Runtime.Decref(self.tpHandle);
+            Runtime.XDecref(self.tpHandle);
             self.gcHandle.Free();
         }
 
@@ -70,9 +61,11 @@ namespace Python.Runtime {
         // Type __setattr__ implementation.
         //====================================================================
 
-        public static int tp_setattro(IntPtr ob, IntPtr key, IntPtr val) {
+        public static int tp_setattro(IntPtr ob, IntPtr key, IntPtr val)
+        {
             string message = "type does not support setting attributes";
-            if (val == IntPtr.Zero) {
+            if (val == IntPtr.Zero)
+            {
                 message = "readonly attribute";
             }
             Exceptions.SetError(Exceptions.TypeError, message);
@@ -85,7 +78,8 @@ namespace Python.Runtime {
         // being silently replaced in a type __dict__ by default __setattr__.
         //====================================================================
 
-        public static int tp_descr_set(IntPtr ds, IntPtr ob, IntPtr val) {
+        public static int tp_descr_set(IntPtr ds, IntPtr ob, IntPtr val)
+        {
             string message = "attribute is read-only";
             Exceptions.SetError(Exceptions.AttributeError, message);
             return -1;
@@ -96,17 +90,20 @@ namespace Python.Runtime {
         // Required Python GC support.
         //====================================================================
 
-        public static int tp_traverse(IntPtr ob, IntPtr func, IntPtr args) {
+        public static int tp_traverse(IntPtr ob, IntPtr func, IntPtr args)
+        {
             return 0;
         }
 
 
-        public static int tp_clear(IntPtr ob) {
+        public static int tp_clear(IntPtr ob)
+        {
             return 0;
         }
 
 
-        public static int tp_is_gc(IntPtr type) {
+        public static int tp_is_gc(IntPtr type)
+        {
             return 1;
         }
 
@@ -115,15 +112,12 @@ namespace Python.Runtime {
         // Default dealloc implementation.
         //====================================================================
 
-        public static void tp_dealloc(IntPtr ob) {
-            // Clean up a Python instance of this extension type. This 
+        public static void tp_dealloc(IntPtr ob)
+        {
+            // Clean up a Python instance of this extension type. This
             // frees the allocated Python object and decrefs the type.
             ManagedType self = GetManagedObject(ob);
             FinalizeObject(self);
         }
-
-
     }
-
-
-}
+}
\ No newline at end of file
