@@ -1,20 +1,12 @@
-// ==========================================================================
-// This software is subject to the provisions of the Zope Public License,
-// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
-// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
-// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-// FOR A PARTICULAR PURPOSE.
-// ==========================================================================
-
 using System;
 using System.Collections;
 using System.Collections.Specialized;
 using System.Runtime.InteropServices;
 using System.Reflection;
+using System.Text;
 
-namespace Python.Runtime {
-
+namespace Python.Runtime
+{
     //=======================================================================
     // This file defines objects to support binary interop with the Python
     // runtime. Generally, the definitions here need to be kept up to date
@@ -23,285 +15,261 @@ namespace Python.Runtime {
 
     [Serializable()]
     [AttributeUsage(AttributeTargets.All)]
-    public class DocStringAttribute : Attribute {
-        public DocStringAttribute(string docStr) {
-			DocString = docStr;
-		}
-		public string DocString {
-			get { return docStr; }
-			set { docStr = value; }
-		}
-		private string docStr;
-    }
-	
-    [Serializable()]
-    [AttributeUsage(AttributeTargets.Method | AttributeTargets.Delegate)]
-    internal class PythonMethodAttribute : Attribute {
-        public PythonMethodAttribute() {}
+    public class DocStringAttribute : Attribute
+    {
+        public DocStringAttribute(string docStr)
+        {
+            DocString = docStr;
+        }
+
+        public string DocString
+        {
+            get { return docStr; }
+            set { docStr = value; }
+        }
+
+        private string docStr;
     }
 
     [Serializable()]
     [AttributeUsage(AttributeTargets.Method | AttributeTargets.Delegate)]
-    internal class ModuleFunctionAttribute : Attribute {
-        public ModuleFunctionAttribute() {}
+    internal class PythonMethodAttribute : Attribute
+    {
+        public PythonMethodAttribute()
+        {
+        }
     }
 
     [Serializable()]
     [AttributeUsage(AttributeTargets.Method | AttributeTargets.Delegate)]
-    internal class ForbidPythonThreadsAttribute : Attribute {
-        public ForbidPythonThreadsAttribute() { }
+    internal class ModuleFunctionAttribute : Attribute
+    {
+        public ModuleFunctionAttribute()
+        {
+        }
+    }
+
+    [Serializable()]
+    [AttributeUsage(AttributeTargets.Method | AttributeTargets.Delegate)]
+    internal class ForbidPythonThreadsAttribute : Attribute
+    {
+        public ForbidPythonThreadsAttribute()
+        {
+        }
     }
 
 
     [Serializable()]
     [AttributeUsage(AttributeTargets.Property)]
-    internal class ModulePropertyAttribute : Attribute {
-        public ModulePropertyAttribute() {}
+    internal class ModulePropertyAttribute : Attribute
+    {
+        public ModulePropertyAttribute()
+        {
+        }
     }
 
 
-    [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
-    internal class ObjectOffset {
-
-        static ObjectOffset() {
+    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
+    internal class ObjectOffset
+    {
+        static ObjectOffset()
+        {
             int size = IntPtr.Size;
-            int n = 0; // Py_TRACE_REFS add two pointers to PyObject_HEAD 
-#if (Py_DEBUG)
+            int n = 0; // Py_TRACE_REFS add two pointers to PyObject_HEAD
+#if Py_DEBUG
             _ob_next = 0;
             _ob_prev = 1 * size;
             n = 2;
-#endif 
-            ob_refcnt = (n+0) * size;
-            ob_type = (n+1) * size;
-            ob_dict = (n+2) * size;
-            ob_data = (n+3) * size;
+#endif
+            ob_refcnt = (n + 0)*size;
+            ob_type = (n + 1)*size;
+            ob_dict = (n + 2)*size;
+            ob_data = (n + 3)*size;
         }
 
-        public static int magic() {
+        public static int magic(IntPtr ob)
+        {
+            if ((Runtime.PyObject_TypeCheck(ob, Exceptions.BaseException) ||
+                (Runtime.PyType_Check(ob) && Runtime.PyType_IsSubtype(ob, Exceptions.BaseException))))
+            {
+                return ExceptionOffset.ob_data;
+            }
             return ob_data;
         }
 
-        public static int Size() {
-#if (Py_DEBUG)
+        public static int DictOffset(IntPtr ob)
+        {
+            if ((Runtime.PyObject_TypeCheck(ob, Exceptions.BaseException) ||
+                (Runtime.PyType_Check(ob) && Runtime.PyType_IsSubtype(ob, Exceptions.BaseException))))
+            {
+                return ExceptionOffset.ob_dict;
+            }
+            return ob_dict;
+        }
+
+        public static int Size(IntPtr ob)
+        {
+            if ((Runtime.PyObject_TypeCheck(ob, Exceptions.BaseException) ||
+                (Runtime.PyType_Check(ob) && Runtime.PyType_IsSubtype(ob, Exceptions.BaseException))))
+            {
+                return ExceptionOffset.Size();
+            }
+#if Py_DEBUG
             return 6 * IntPtr.Size;
 #else
             return 4 * IntPtr.Size;
 #endif
         }
 
-#if (Py_DEBUG)
+#if Py_DEBUG
         public static int _ob_next;
         public static int _ob_prev;
 #endif
         public static int ob_refcnt;
         public static int ob_type;
+        private static int ob_dict;
+        private static int ob_data;
+    }
+
+    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
+    internal class ExceptionOffset
+    {
+        static ExceptionOffset()
+        {
+            Type type = typeof(ExceptionOffset);
+            FieldInfo[] fi = type.GetFields();
+            int size = IntPtr.Size;
+            for (int i = 0; i < fi.Length; i++)
+            {
+                fi[i].SetValue(null, (i * size) + ObjectOffset.ob_type + size);
+            }
+        }
+
+        public static int Size()
+        {
+            return ob_data + IntPtr.Size;
+        }
+
+        // PyException_HEAD
+        // (start after PyObject_HEAD)
+        public static int dict = 0;
+        public static int args = 0;
+#if PYTHON2
+        public static int message = 0;
+#elif PYTHON3
+        public static int traceback = 0;
+        public static int context = 0;
+        public static int cause = 0;
+        public static int suppress_context = 0;
+#endif
+
+        // extra c# data
         public static int ob_dict;
         public static int ob_data;
     }
 
 
-    [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
-    internal class TypeOffset {
-
-        static TypeOffset() {
-            Type type = typeof(TypeOffset);
+#if PYTHON3
+    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
+    internal class BytesOffset
+    {
+        static BytesOffset()
+        {
+            Type type = typeof(BytesOffset);
             FieldInfo[] fi = type.GetFields();
             int size = IntPtr.Size;
-            for (int i = 0; i < fi.Length; i++) {
+            for (int i = 0; i < fi.Length; i++)
+            {
                 fi[i].SetValue(null, i * size);
             }
         }
 
-        public static int magic() {
-            return ob_size;
-        }
-        
-/* The *real* layout of a type object when allocated on the heap */
-//typedef struct _heaptypeobject {
-#if (Py_DEBUG)  // #ifdef Py_TRACE_REFS
+        /* The *real* layout of a type object when allocated on the heap */
+        //typedef struct _heaptypeobject {
+#if Py_DEBUG  // #ifdef Py_TRACE_REFS
 /* _PyObject_HEAD_EXTRA defines pointers to support a doubly-linked list of all live heap objects. */
         public static int _ob_next = 0;
         public static int _ob_prev = 0;
 #endif
-// PyObject_VAR_HEAD {
-//     PyObject_HEAD {
+        // PyObject_VAR_HEAD {
+        //     PyObject_HEAD {
         public static int ob_refcnt = 0;
         public static int ob_type = 0;
-    // }
+        // }
         public static int ob_size = 0;      /* Number of items in _VAR_iable part */
-// }
-        public static int tp_name = 0;      /* For printing, in format "<module>.<name>" */
-        public static int tp_basicsize = 0; /* For allocation */
-        public static int tp_itemsize = 0;
+        // }
+        public static int ob_shash = 0;
+        public static int ob_sval = 0; /* start of data */
 
-        /* Methods to implement standard operations */
-        public static int tp_dealloc = 0;
-        public static int tp_print = 0;
-        public static int tp_getattr = 0;
-        public static int tp_setattr = 0;
-        public static int tp_compare = 0;
-        public static int tp_repr = 0;
-
-        /* Method suites for standard classes */
-        public static int tp_as_number = 0;
-        public static int tp_as_sequence = 0;
-        public static int tp_as_mapping = 0;
-
-        /* More standard operations (here for binary compatibility) */
-        public static int tp_hash = 0;
-        public static int tp_call = 0;
-        public static int tp_str = 0;
-        public static int tp_getattro = 0;
-        public static int tp_setattro = 0;
-
-        /* Functions to access object as input/output buffer */
-        public static int tp_as_buffer = 0;
-
-        /* Flags to define presence of optional/expanded features */
-        public static int tp_flags = 0;
-
-        public static int tp_doc = 0; /* Documentation string */
-
-        /* Assigned meaning in release 2.0 */
-        /* call function for all accessible objects */
-        public static int tp_traverse = 0;
-        /* delete references to contained objects */
-        public static int tp_clear = 0;
-
-        /* Assigned meaning in release 2.1 */
-        /* rich comparisons */
-        public static int tp_richcompare = 0;
-        /* weak reference enabler */
-        public static int tp_weaklistoffset = 0;
-
-        /* Added in release 2.2 */
-        /* Iterators */
-        public static int tp_iter = 0;
-        public static int tp_iternext = 0;
-        /* Attribute descriptor and subclassing stuff */
-        public static int tp_methods = 0;
-        public static int tp_members = 0;
-        public static int tp_getset = 0;
-        public static int tp_base = 0;
-        public static int tp_dict = 0;
-        public static int tp_descr_get = 0;
-        public static int tp_descr_set = 0;
-        public static int tp_dictoffset = 0;
-        public static int tp_init = 0;
-        public static int tp_alloc = 0;
-        public static int tp_new = 0;
-        public static int tp_free = 0;      /* Low-level free-memory routine */
-        public static int tp_is_gc = 0;     /* For PyObject_IS_GC */
-        public static int tp_bases = 0;
-        public static int tp_mro = 0;       /* method resolution order */
-        public static int tp_cache = 0;
-        public static int tp_subclasses = 0;
-        public static int tp_weaklist = 0;
-        public static int tp_del = 0;
-#if (PYTHON26 || PYTHON27)
-        /* Type attribute cache version tag. Added in version 2.6 */
-	    public static int tp_version_tag;
-#endif
-        // COUNT_ALLOCS adds some more stuff to PyTypeObject 
-#if (Py_COUNT_ALLOCS)
-	/* these must be last and never explicitly initialized */
-        public static int tp_allocs = 0;
-        public static int tp_frees = 0;
-        public static int tp_maxalloc = 0;
-        public static int tp_prev = 0;
-        public static int tp_next = 0;
-#endif
-//} PyTypeObject;
-//typedef struct {
-        public static int nb_add = 0;
-        public static int nb_subtract = 0;
-        public static int nb_multiply = 0;
-        public static int nb_divide = 0;
-        public static int nb_remainder = 0;
-        public static int nb_divmod = 0;
-        public static int nb_power = 0;
-        public static int nb_negative = 0;
-        public static int nb_positive = 0;
-        public static int nb_absolute = 0;
-        public static int nb_nonzero = 0;
-        public static int nb_invert = 0;
-        public static int nb_lshift = 0;
-        public static int nb_rshift = 0;
-        public static int nb_and = 0;
-        public static int nb_xor = 0;
-        public static int nb_or = 0;
-        public static int nb_coerce = 0;
-        public static int nb_int = 0;
-        public static int nb_long = 0;
-        public static int nb_float = 0;
-        public static int nb_oct = 0;
-        public static int nb_hex = 0;
-        /* Added in release 2.0 */
-        public static int nb_inplace_add = 0;
-        public static int nb_inplace_subtract = 0;
-        public static int nb_inplace_multiply = 0;
-        public static int nb_inplace_divide = 0;
-        public static int nb_inplace_remainder = 0;
-        public static int nb_inplace_power = 0;
-        public static int nb_inplace_lshift = 0;
-        public static int nb_inplace_rshift = 0;
-        public static int nb_inplace_and = 0;
-        public static int nb_inplace_xor = 0;
-        public static int nb_inplace_or = 0;
-        /* Added in release 2.2 */
-        /* The following require the Py_TPFLAGS_HAVE_CLASS flag */
-        public static int nb_floor_divide = 0;
-        public static int nb_true_divide = 0;
-        public static int nb_inplace_floor_divide = 0;
-        public static int nb_inplace_true_divide = 0;
-#if (PYTHON25 || PYTHON26 || PYTHON27)
-        /* Added in release 2.5 */
-        public static int nb_index = 0;
-#endif
-        //} PyNumberMethods;
-//typedef struct {
-        public static int mp_length = 0;
-        public static int mp_subscript = 0;
-        public static int mp_ass_subscript = 0;
-//} PyMappingMethods;
-//typedef struct {
-        public static int sq_length = 0;
-        public static int sq_concat = 0;
-        public static int sq_repeat = 0;
-        public static int sq_item = 0;
-        public static int sq_slice = 0;
-        public static int sq_ass_item = 0;
-        public static int sq_ass_slice = 0;
-        public static int sq_contains = 0;
-        /* Added in release 2.0 */
-        public static int sq_inplace_concat = 0;
-        public static int sq_inplace_repeat = 0;
-//} PySequenceMethods;
-//typedef struct {
-        public static int bf_getreadbuffer = 0;
-        public static int bf_getwritebuffer = 0;
-        public static int bf_getsegcount = 0;
-        public static int bf_getcharbuffer = 0;
-#if (PYTHON26 || PYTHON27)
-        // This addition is not actually noted in the 2.6.5 object.h
-	    public static int bf_getbuffer = 0;
-	    public static int bf_releasebuffer = 0;
-//} PyBufferProcs;
-#endif
-        //PyObject *ht_name, *ht_slots;
-        public static int name = 0;
-        public static int slots = 0;
-        /* here are optional user slots, followed by the members. */
-        public static int members = 0;
+        /* Invariants:
+         *     ob_sval contains space for 'ob_size+1' elements.
+         *     ob_sval[ob_size] == 0.
+         *     ob_shash is the hash of the string or -1 if not computed yet.
+         */
+        //} PyBytesObject;
     }
 
+    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
+    internal class ModuleDefOffset
+    {
+        static ModuleDefOffset()
+        {
+            Type type = typeof(ModuleDefOffset);
+            FieldInfo[] fi = type.GetFields();
+            int size = IntPtr.Size;
+            for (int i = 0; i < fi.Length; i++)
+            {
+                fi[i].SetValue(null, (i * size) + TypeOffset.ob_size);
+            }
+        }
+
+        public static IntPtr AllocModuleDef(string modulename) {
+            byte[] ascii = Encoding.ASCII.GetBytes(modulename);
+            int size = name + ascii.Length + 1;
+            IntPtr ptr = Marshal.AllocHGlobal(size);
+            for (int i = 0; i <= m_free; i += IntPtr.Size)
+                Marshal.WriteIntPtr(ptr, i, IntPtr.Zero);
+            Marshal.Copy(ascii, 0, (IntPtr)(ptr + name), ascii.Length);
+            Marshal.WriteIntPtr(ptr, m_name, (IntPtr)(ptr + name));
+            Marshal.WriteByte(ptr, name + ascii.Length, 0);
+            return ptr;
+        }
+
+        public static void FreeModuleDef(IntPtr ptr) {
+            Marshal.FreeHGlobal(ptr);
+        }
+
+        // typedef struct PyModuleDef{
+        //  typedef struct PyModuleDef_Base {
+        // starts after PyObject_HEAD (TypeOffset.ob_type + 1)
+        public static int m_init = 0;
+        public static int m_index = 0;
+        public static int m_copy = 0;
+        //  } PyModuleDef_Base
+        public static int m_name = 0;
+        public static int m_doc = 0;
+        public static int m_size = 0;
+        public static int m_methods = 0;
+        public static int m_reload = 0;
+        public static int m_traverse = 0;
+        public static int m_clear = 0;
+        public static int m_free = 0;
+        // } PyModuleDef
+
+        public static int name = 0;
+    }
+#endif // PYTHON3
+
     /// <summary>
     /// TypeFlags(): The actual bit values for the Type Flags stored
     /// in a class.
     /// Note that the two values reserved for stackless have been put
     /// to good use as PythonNet specific flags (Managed and Subclass)
     /// </summary>
-    internal class TypeFlags {
+    internal class TypeFlags
+    {
+#if PYTHON2
+        // these flags were removed in Python 3
         public static int HaveGetCharBuffer = (1 << 0);
         public static int HaveSequenceIn = (1 << 1);
         public static int GC = 0;
@@ -311,6 +279,7 @@ namespace Python.Runtime {
         public static int HaveWeakRefs = (1 << 6);
         public static int HaveIter = (1 << 7);
         public static int HaveClass = (1 << 8);
+#endif
         public static int HeapType = (1 << 9);
         public static int BaseType = (1 << 10);
         public static int Ready = (1 << 12);
@@ -321,10 +290,7 @@ namespace Python.Runtime {
         /* XXX Reusing reserved constants */
         public static int Managed = (1 << 15); // PythonNet specific
         public static int Subclass = (1 << 16); // PythonNet specific
-#if (PYTHON25 || PYTHON26 || PYTHON27)
         public static int HaveIndex = (1 << 17);
-#endif
-#if (PYTHON26 || PYTHON27)
         /* Objects support nb_index in PyNumberMethods */
         public static int HaveVersionTag = (1 << 18);
         public static int ValidVersionTag = (1 << 19);
@@ -340,41 +306,49 @@ namespace Python.Runtime {
         public static int DictSubclass = (1 << 29);
         public static int BaseExceptionSubclass = (1 << 30);
         public static int TypeSubclass = (1 << 31);
+
+#if PYTHON2
+        // Default flags for Python 2
+        public static int Default = (
+            HaveGetCharBuffer |
+            HaveSequenceIn |
+            HaveInPlaceOps |
+            HaveRichCompare |
+            HaveWeakRefs |
+            HaveIter |
+            HaveClass |
+            HaveStacklessExtension |
+                HaveIndex |
+                0);
+#elif PYTHON3
+        // Default flags for Python 3
+        public static int Default = (
+                            HaveStacklessExtension |
+                            HaveVersionTag);
 #endif
-        public static int Default = (HaveGetCharBuffer |
-                             HaveSequenceIn |
-                             HaveInPlaceOps |
-                             HaveRichCompare |
-                             HaveWeakRefs |
-                             HaveIter |
-                             HaveClass |
-                             HaveStacklessExtension |
-#if (PYTHON25 || PYTHON26 || PYTHON27)
-                             HaveIndex | 
-#endif
-                             0);
     }
 
 
     // This class defines the function prototypes (delegates) used for low
-    // level integration with the CPython runtime. It also provides name 
-    // based lookup of the correct prototype for a particular Python type 
+    // level integration with the CPython runtime. It also provides name
+    // based lookup of the correct prototype for a particular Python type
     // slot and utilities for generating method thunks for managed methods.
 
-    internal class Interop {
-
+    internal class Interop
+    {
         static ArrayList keepAlive;
         static Hashtable pmap;
 
-        static Interop() {
-
+        static Interop()
+        {
             // Here we build a mapping of PyTypeObject slot names to the
             // appropriate prototype (delegate) type to use for the slot.
 
             Type[] items = typeof(Interop).GetNestedTypes();
             Hashtable p = new Hashtable();
 
-            for (int i = 0; i < items.Length; i++) {
+            for (int i = 0; i < items.Length; i++)
+            {
                 Type item = items[i];
                 p[item.Name] = item;
             }
@@ -410,7 +384,9 @@ namespace Python.Runtime {
             pmap["nb_add"] = p["BinaryFunc"];
             pmap["nb_subtract"] = p["BinaryFunc"];
             pmap["nb_multiply"] = p["BinaryFunc"];
+#if PYTHON2
             pmap["nb_divide"] = p["BinaryFunc"];
+#endif
             pmap["nb_remainder"] = p["BinaryFunc"];
             pmap["nb_divmod"] = p["BinaryFunc"];
             pmap["nb_power"] = p["TernaryFunc"];
@@ -433,7 +409,9 @@ namespace Python.Runtime {
             pmap["nb_inplace_add"] = p["BinaryFunc"];
             pmap["nb_inplace_subtract"] = p["BinaryFunc"];
             pmap["nb_inplace_multiply"] = p["BinaryFunc"];
+#if PYTHON2
             pmap["nb_inplace_divide"] = p["BinaryFunc"];
+#endif
             pmap["nb_inplace_remainder"] = p["BinaryFunc"];
             pmap["nb_inplace_power"] = p["TernaryFunc"];
             pmap["nb_inplace_lshift"] = p["BinaryFunc"];
@@ -445,9 +423,7 @@ namespace Python.Runtime {
             pmap["nb_true_divide"] = p["BinaryFunc"];
             pmap["nb_inplace_floor_divide"] = p["BinaryFunc"];
             pmap["nb_inplace_true_divide"] = p["BinaryFunc"];
-#if (PYTHON25 || PYTHON26 || PYTHON27)
             pmap["nb_index"] = p["UnaryFunc"];
-#endif
 
             pmap["sq_length"] = p["InquiryFunc"];
             pmap["sq_concat"] = p["BinaryFunc"];
@@ -463,22 +439,28 @@ namespace Python.Runtime {
             pmap["mp_length"] = p["InquiryFunc"];
             pmap["mp_subscript"] = p["BinaryFunc"];
             pmap["mp_ass_subscript"] = p["ObjObjArgFunc"];
-            
+
             pmap["bf_getreadbuffer"] = p["IntObjArgFunc"];
             pmap["bf_getwritebuffer"] = p["IntObjArgFunc"];
             pmap["bf_getsegcount"] = p["ObjObjFunc"];
             pmap["bf_getcharbuffer"] = p["IntObjArgFunc"];
-
-            pmap["__import__"] = p["TernaryFunc"];
         }
 
-        internal static Type GetPrototype(string name) {
+        internal static Type GetPrototype(string name)
+        {
             return pmap[name] as Type;
         }
 
-        internal static IntPtr GetThunk(MethodInfo method) {
-            Type dt = Interop.GetPrototype(method.Name);
-            if (dt != null) {
+        internal static IntPtr GetThunk(MethodInfo method, string funcType = null)
+        {
+            Type dt;
+            if (funcType != null)
+                dt = typeof(Interop).GetNestedType(funcType) as Type;
+            else
+                dt = GetPrototype(method.Name);
+
+            if (dt != null)
+            {
                 IntPtr tmp = Marshal.AllocHGlobal(IntPtr.Size);
                 Delegate d = Delegate.CreateDelegate(dt, method);
                 Thunk cb = new Thunk(d);
@@ -529,17 +511,17 @@ namespace Python.Runtime {
 
         [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
         public delegate IntPtr RichCmpFunc(IntPtr ob, IntPtr a, int b);
-
     }
 
 
-    [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
-    internal struct Thunk {
+    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
+    internal struct Thunk
+    {
         public Delegate fn;
 
-        public Thunk(Delegate d) {
+        public Thunk(Delegate d)
+        {
             fn = d;
         }
     }
-
 }
