@@ -1,12 +1,3 @@
-// ==========================================================================
-// This software is subject to the provisions of the Zope Public License,
-// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
-// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
-// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-// FOR A PARTICULAR PURPOSE.
-// ==========================================================================
-
 using System;
 using System.Collections.Specialized;
 using System.Runtime.InteropServices;
@@ -14,21 +5,22 @@ using System.Collections.Generic;
 using System.Collections;
 using System.Reflection;
 
-namespace Python.Runtime {
-
+namespace Python.Runtime
+{
     //========================================================================
-    // Implements a Python type that provides access to CLR namespaces. The 
+    // Implements a Python type that provides access to CLR namespaces. The
     // type behaves like a Python module, and can contain other sub-modules.
     //========================================================================
 
-    internal class ModuleObject : ExtensionType {
-
+    internal class ModuleObject : ExtensionType
+    {
         Dictionary<string, ManagedType> cache;
         internal string moduleName;
         internal IntPtr dict;
         protected string _namespace;
 
-        public ModuleObject(string name) : base() {
+        public ModuleObject(string name) : base()
+        {
             if (name == String.Empty)
             {
                 throw new ArgumentException("Name must not be empty!");
@@ -37,14 +29,33 @@ namespace Python.Runtime {
             cache = new Dictionary<string, ManagedType>();
             _namespace = name;
 
+            // Use the filename from any of the assemblies just so there's something for
+            // anything that expects __file__ to be set.
+            string filename = "unknown";
+            string docstring = "Namespace containing types from the following assemblies:\n\n";
+            foreach (Assembly a in AssemblyManager.GetAssemblies(name))
+            {
+                if (!a.IsDynamic && a.Location != null)
+                {
+                    filename = a.Location;
+                }
+                docstring += "- " + a.FullName + "\n";
+            }
+
             dict = Runtime.PyDict_New();
             IntPtr pyname = Runtime.PyString_FromString(moduleName);
+            IntPtr pyfilename = Runtime.PyString_FromString(filename);
+            IntPtr pydocstring = Runtime.PyString_FromString(docstring);
+            IntPtr pycls = TypeManager.GetTypeHandle(this.GetType());
             Runtime.PyDict_SetItemString(dict, "__name__", pyname);
-            Runtime.PyDict_SetItemString(dict, "__file__", Runtime.PyNone);
-            Runtime.PyDict_SetItemString(dict, "__doc__", Runtime.PyNone);
-            Runtime.Decref(pyname);
+            Runtime.PyDict_SetItemString(dict, "__file__", pyfilename);
+            Runtime.PyDict_SetItemString(dict, "__doc__", pydocstring);
+            Runtime.PyDict_SetItemString(dict, "__class__", pycls);
+            Runtime.XDecref(pyname);
+            Runtime.XDecref(pyfilename);
+            Runtime.XDecref(pydocstring);
 
-            Marshal.WriteIntPtr(this.pyHandle, ObjectOffset.ob_dict, dict);
+            Marshal.WriteIntPtr(this.pyHandle, ObjectOffset.DictOffset(this.pyHandle), dict);
 
             InitializeModuleMembers();
         }
@@ -52,15 +63,17 @@ namespace Python.Runtime {
 
         //===================================================================
         // Returns a ClassBase object representing a type that appears in
-        // this module's namespace or a ModuleObject representing a child 
+        // this module's namespace or a ModuleObject representing a child
         // namespace (or null if the name is not found). This method does
         // not increment the Python refcount of the returned object.
         //===================================================================
 
-        public ManagedType GetAttribute(string name, bool guess) {
+        public ManagedType GetAttribute(string name, bool guess)
+        {
             ManagedType cached = null;
             this.cache.TryGetValue(name, out cached);
-            if (cached != null) {
+            if (cached != null)
+            {
                 return cached;
             }
 
@@ -78,57 +91,65 @@ namespace Python.Runtime {
             //    return null;
             //}
 
-            string qname = (_namespace == String.Empty) ? name : 
-                            _namespace + "." + name;
+            string qname = (_namespace == String.Empty)
+                ? name
+                : _namespace + "." + name;
 
-            // If the fully-qualified name of the requested attribute is 
-            // a namespace exported by a currently loaded assembly, return 
+            // If the fully-qualified name of the requested attribute is
+            // a namespace exported by a currently loaded assembly, return
             // a new ModuleObject representing that namespace.
 
-            if (AssemblyManager.IsValidNamespace(qname)) {
+            if (AssemblyManager.IsValidNamespace(qname))
+            {
                 m = new ModuleObject(qname);
                 StoreAttribute(name, m);
-                return (ManagedType) m;
+                return (ManagedType)m;
             }
 
-            // Look for a type in the current namespace. Note that this 
+            // Look for a type in the current namespace. Note that this
             // includes types, delegates, enums, interfaces and structs.
             // Only public namespace members are exposed to Python.
 
             type = AssemblyManager.LookupType(qname);
-            if (type != null) {
-                if (!type.IsPublic) {
+            if (type != null)
+            {
+                if (!type.IsPublic)
+                {
                     return null;
                 }
                 c = ClassManager.GetClass(type);
                 StoreAttribute(name, c);
-                return (ManagedType) c;
+                return (ManagedType)c;
             }
 
             // This is a little repetitive, but it ensures that the right
             // thing happens with implicit assembly loading at a reasonable
-            // cost. Ask the AssemblyManager to do implicit loading for each 
+            // cost. Ask the AssemblyManager to do implicit loading for each
             // of the steps in the qualified name, then try it again.
             bool ignore = name.StartsWith("__");
-            if (AssemblyManager.LoadImplicit(qname, !ignore)) {
-                if (AssemblyManager.IsValidNamespace(qname)) {
+            if (AssemblyManager.LoadImplicit(qname, !ignore))
+            {
+                if (AssemblyManager.IsValidNamespace(qname))
+                {
                     m = new ModuleObject(qname);
                     StoreAttribute(name, m);
-                    return (ManagedType) m;
+                    return (ManagedType)m;
                 }
 
                 type = AssemblyManager.LookupType(qname);
-                if (type != null) {
-                    if (!type.IsPublic) {
+                if (type != null)
+                {
+                    if (!type.IsPublic)
+                    {
                         return null;
                     }
                     c = ClassManager.GetClass(type);
                     StoreAttribute(name, c);
-                    return (ManagedType) c;
+                    return (ManagedType)c;
                 }
             }
 
-            // We didn't find the name, so we may need to see if there is a 
+            // We didn't find the name, so we may need to see if there is a
             // generic type with this base name. If so, we'll go ahead and
             // return it. Note that we store the mapping of the unmangled
             // name to generic type -  it is technically possible that some
@@ -136,12 +157,15 @@ namespace Python.Runtime {
             // the current namespace with the given basename, but unlikely
             // enough to complicate the implementation for now.
 
-            if (guess) {
+            if (guess)
+            {
                 string gname = GenericUtil.GenericNameForBaseName(
-                                              _namespace, name);
-                if (gname != null) {
+                    _namespace, name);
+                if (gname != null)
+                {
                     ManagedType o = GetAttribute(gname, false);
-                    if (o != null) {
+                    if (o != null)
+                    {
                         StoreAttribute(name, o);
                         return o;
                     }
@@ -154,9 +178,10 @@ namespace Python.Runtime {
 
         //===================================================================
         // Stores an attribute in the instance dict for future lookups.
-         //===================================================================
+        //===================================================================
 
-        private void StoreAttribute(string name, ManagedType ob) {
+        private void StoreAttribute(string name, ManagedType ob)
+        {
             Runtime.PyDict_SetItemString(dict, name, ob.pyHandle);
             cache[name] = ob;
         }
@@ -166,26 +191,17 @@ namespace Python.Runtime {
         // Preloads all currently-known names for the module namespace. This
         // can be called multiple times, to add names from assemblies that
         // may have been loaded since the last call to the method.
-         //===================================================================
+        //===================================================================
 
-        public void LoadNames() {
+        public void LoadNames()
+        {
             ManagedType m = null;
-            foreach (string name in AssemblyManager.GetNames(_namespace)) {
+            foreach (string name in AssemblyManager.GetNames(_namespace))
+            {
                 this.cache.TryGetValue(name, out m);
-                if (m == null) {
+                if (m == null)
+                {
                     ManagedType attr = this.GetAttribute(name, true);
-                    if (Runtime.wrap_exceptions) {
-                        if (attr is ExceptionClassObject) {
-                            ExceptionClassObject c = attr as ExceptionClassObject;
-                            if (c != null) {
-                              IntPtr p = attr.pyHandle;
-                              IntPtr r =Exceptions.GetExceptionClassWrapper(p);
-                              Runtime.PyDict_SetItemString(dict, name, r);
-                              Runtime.Incref(r);
-
-                            }
-                        }
-                    }
                 }
             }
         }
@@ -216,7 +232,7 @@ namespace Python.Runtime {
                         string name = method.Name;
                         MethodInfo[] mi = new MethodInfo[1];
                         mi[0] = method;
-                        ModuleFunctionObject m = new ModuleFunctionObject(name, mi, allow_threads);
+                        ModuleFunctionObject m = new ModuleFunctionObject(type, name, mi, allow_threads);
                         StoreAttribute(name, m);
                     }
                 }
@@ -240,56 +256,44 @@ namespace Python.Runtime {
 
         //====================================================================
         // ModuleObject __getattribute__ implementation. Module attributes
-        // are always either classes or sub-modules representing subordinate 
+        // are always either classes or sub-modules representing subordinate
         // namespaces. CLR modules implement a lazy pattern - the sub-modules
         // and classes are created when accessed and cached for future use.
         //====================================================================
 
-        public static IntPtr tp_getattro(IntPtr ob, IntPtr key) {
+        public static IntPtr tp_getattro(IntPtr ob, IntPtr key)
+        {
             ModuleObject self = (ModuleObject)GetManagedObject(ob);
 
-            if (!Runtime.PyString_Check(key)) {
+            if (!Runtime.PyString_Check(key))
+            {
                 Exceptions.SetError(Exceptions.TypeError, "string expected");
                 return IntPtr.Zero;
             }
 
             IntPtr op = Runtime.PyDict_GetItem(self.dict, key);
-            if (op != IntPtr.Zero) {
-                Runtime.Incref(op);
+            if (op != IntPtr.Zero)
+            {
+                Runtime.XIncref(op);
                 return op;
             }
- 
+
             string name = Runtime.GetManagedString(key);
-            if (name == "__dict__") {
-                Runtime.Incref(self.dict);
+            if (name == "__dict__")
+            {
+                Runtime.XIncref(self.dict);
                 return self.dict;
             }
 
             ManagedType attr = self.GetAttribute(name, true);
 
-            if (attr == null) {
+            if (attr == null)
+            {
                 Exceptions.SetError(Exceptions.AttributeError, name);
-                return IntPtr.Zero;                
+                return IntPtr.Zero;
             }
 
-            // XXX - hack required to recognize exception types. These types
-            // may need to be wrapped in old-style class wrappers in versions
-            // of Python where new-style classes cannot be used as exceptions.
-
-            if (Runtime.wrap_exceptions) {
-                if (attr is ExceptionClassObject) {
-                    ExceptionClassObject c = attr as ExceptionClassObject;
-                    if (c != null) {
-                        IntPtr p = attr.pyHandle;
-                        IntPtr r = Exceptions.GetExceptionClassWrapper(p);
-                        Runtime.PyDict_SetItemString(self.dict, name, r);
-                        Runtime.Incref(r);
-                        return r;
-                    }
-                }
-            }
-
-            Runtime.Incref(attr.pyHandle);
+            Runtime.XIncref(attr.pyHandle);
             return attr.pyHandle;
         }
 
@@ -297,14 +301,12 @@ namespace Python.Runtime {
         // ModuleObject __repr__ implementation.
         //====================================================================
 
-        public static IntPtr tp_repr(IntPtr ob) {
+        public static IntPtr tp_repr(IntPtr ob)
+        {
             ModuleObject self = (ModuleObject)GetManagedObject(ob);
             string s = String.Format("<module '{0}'>", self.moduleName);
             return Runtime.PyString_FromString(s);
         }
-
-
-
     }
 
     /// <summary>
@@ -321,11 +323,12 @@ namespace Python.Runtime {
         internal static bool _SuppressDocs = false;
         internal static bool _SuppressOverloads = false;
 
-        public CLRModule() : base("clr") {
+        public CLRModule() : base("clr")
+        {
             _namespace = String.Empty;
-            
+
             // This hackery is required in order to allow a plain Python to
-            // import the managed runtime via the CLR bootstrapper module. 
+            // import the managed runtime via the CLR bootstrapper module.
             // The standard Python machinery in control at the time of the
             // import requires the module to pass PyModule_Check. :(
             if (!hacked)
@@ -334,22 +337,27 @@ namespace Python.Runtime {
                 IntPtr mro = Marshal.ReadIntPtr(type, TypeOffset.tp_mro);
                 IntPtr ext = Runtime.ExtendTuple(mro, Runtime.PyModuleType);
                 Marshal.WriteIntPtr(type, TypeOffset.tp_mro, ext);
-                Runtime.Decref(mro);
+                Runtime.XDecref(mro);
                 hacked = true;
             }
         }
 
         /// <summary>
         /// The initializing of the preload hook has to happen as late as
-        /// possible since sys.ps1 is created after the CLR module is 
+        /// possible since sys.ps1 is created after the CLR module is
         /// created.
         /// </summary>
-        internal void InitializePreload() {
-            if (interactive_preload) {
+        internal void InitializePreload()
+        {
+            if (interactive_preload)
+            {
                 interactive_preload = false;
-                if (Runtime.PySys_GetObject("ps1") != IntPtr.Zero) {
+                if (Runtime.PySys_GetObject("ps1") != IntPtr.Zero)
+                {
                     preload = true;
-                } else {
+                }
+                else
+                {
                     Exceptions.Clear();
                     preload = false;
                 }
@@ -357,7 +365,8 @@ namespace Python.Runtime {
         }
 
         [ModuleFunctionAttribute()]
-        public static bool getPreload() {
+        public static bool getPreload()
+        {
             return preload;
         }
 
@@ -368,13 +377,15 @@ namespace Python.Runtime {
         }
 
         //[ModulePropertyAttribute]
-        public static bool SuppressDocs {
+        public static bool SuppressDocs
+        {
             get { return _SuppressDocs; }
             set { _SuppressDocs = value; }
         }
 
         //[ModulePropertyAttribute]
-        public static bool SuppressOverloads {
+        public static bool SuppressOverloads
+        {
             get { return _SuppressOverloads; }
             set { _SuppressOverloads = value; }
         }
@@ -395,11 +406,16 @@ namespace Python.Runtime {
                 assembly = AssemblyManager.LoadAssembly(name);
             }
             if (assembly == null)
+            {
+                assembly = AssemblyManager.LoadAssemblyFullPath(name);
+            }
+            if (assembly == null)
             {
                 string msg = String.Format("Unable to find assembly '{0}'.", name);
                 throw new System.IO.FileNotFoundException(msg);
             }
-            return assembly ;
+
+            return assembly;
         }
 
         [ModuleFunctionAttribute()]
@@ -425,6 +441,10 @@ namespace Python.Runtime {
             return names;
         }
 
+        [ModuleFunctionAttribute()]
+        public static int _AtExit()
+        {
+            return Runtime.AtExit();
+        }
     }
-
-}
+}
\ No newline at end of file
