@@ -1,16 +1,17 @@
-# ===========================================================================
-# This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
-# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
-# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE.
-# ===========================================================================
-from System.Collections import Hashtable
-from Python.Test import ClassTest
-import sys, os, string, unittest, types
+import clr
+import types
+import unittest
+
 import Python.Test as Test
 import System
+import six
+from Python.Test import ClassTest
+from System.Collections import Hashtable
+
+if six.PY3:
+    DictProxyType = type(object.__dict__)
+else:
+    DictProxyType = types.DictProxyType
 
 
 class ClassTests(unittest.TestCase):
@@ -21,39 +22,33 @@ class ClassTests(unittest.TestCase):
         String = System.String
         self.assertEquals(String.Empty, "")
 
-
     def testBasicValueType(self):
         """Test usage of CLR defined value types."""
         Int32 = System.Int32
         self.assertEquals(Int32.MaxValue, 2147483647)
 
-
     def testClassStandardAttrs(self):
         """Test standard class attributes."""
         self.assertTrue(ClassTest.__name__ == 'ClassTest')
         self.assertTrue(ClassTest.__module__ == 'Python.Test')
-        self.assertTrue(type(ClassTest.__dict__) == types.DictProxyType)
+        self.assertTrue(type(ClassTest.__dict__) == DictProxyType)
         self.assertTrue(len(ClassTest.__doc__) > 0)
 
-
     def testClassDocstrings(self):
         """Test standard class docstring generation"""
         value = 'Void .ctor()'
         self.assertTrue(ClassTest.__doc__ == value)
 
-
     def testClassDefaultStr(self):
         """Test the default __str__ implementation for managed objects."""
         s = System.String("this is a test")
         self.assertTrue(str(s) == "this is a test")
 
-
     def testClassDefaultRepr(self):
         """Test the default __repr__ implementation for managed objects."""
         s = System.String("this is a test")
         self.assertTrue(repr(s).startswith("<System.String object"))
 
-
     def testNonPublicClass(self):
         """Test that non-public classes are inaccessible."""
         from Python import Test
@@ -68,7 +63,6 @@ class ClassTests(unittest.TestCase):
 
         self.assertRaises(AttributeError, test)
 
-
     def testBasicSubclass(self):
         """Test basic subclass of a managed class."""
 
@@ -87,17 +81,16 @@ class ClassTests(unittest.TestCase):
         self.assertTrue(table.Count == 0)
 
         table.set_Item('one', 'one')
-        
+
         self.assertTrue(table.howMany() == 1)
         self.assertTrue(table.Count == 1)
 
         MyTable = None
 
-
     def testSubclassWithNoArgConstructor(self):
         """Test subclass of a managed class with a no-arg constructor."""
         from Python.Test import ClassCtorTest1
-        
+
         class SubClass(ClassCtorTest1):
             def __init__(self, name):
                 self.name = name
@@ -108,7 +101,7 @@ class ClassTests(unittest.TestCase):
     def testSubclassWithVariousConstructors(self):
         """Test subclass of a managed class with various constructors."""
         from Python.Test import ClassCtorTest2
-        
+
         class SubClass(ClassCtorTest2):
             def __init__(self, v):
                 ClassCtorTest2.__init__(self)
@@ -125,7 +118,6 @@ class ClassTests(unittest.TestCase):
         inst = SubClass2('test')
         self.assertTrue(inst.value == 'test')
 
-
     def testStructConstruction(self):
         """Test construction of structs."""
         from System.Drawing import Point
@@ -144,13 +136,11 @@ class ClassTests(unittest.TestCase):
         self.assertTrue(p.X == 10)
         self.assertTrue(p.Y == 10)
 
-
-
     # test strange __new__ interactions
-    
+
     # test weird metatype
     # test recursion
-    # test 
+    # test
 
 
     def testIEnumerableIteration(self):
@@ -166,7 +156,6 @@ class ClassTests(unittest.TestCase):
             cname = item.__class__.__name__
             self.assertTrue(cname.endswith('DictionaryEntry'))
 
-
     def testIEnumeratorIteration(self):
         """Test iteration over objects supporting IEnumerator."""
         chars = Test.ClassTest.GetEnumerator()
@@ -174,14 +163,10 @@ class ClassTests(unittest.TestCase):
         for item in chars:
             self.assertTrue(item in 'test string')
 
-
-
-
     def testOverrideGetItem(self):
         """Test managed subclass overriding __getitem__."""
 
         class MyTable(Hashtable):
-                
             def __getitem__(self, key):
                 value = Hashtable.__getitem__(self, key)
                 return 'my ' + str(value)
@@ -195,14 +180,12 @@ class ClassTests(unittest.TestCase):
         self.assertTrue(table['two'] == 'my two')
         self.assertTrue(table['three'] == 'my three')
 
-        self.assertTrue(table.Count == 3)                    
-
+        self.assertTrue(table.Count == 3)
 
     def testOverrideSetItem(self):
         """Test managed subclass overriding __setitem__."""
 
         class MyTable(Hashtable):
-                
             def __setitem__(self, key, value):
                 value = 'my ' + str(value)
                 Hashtable.__setitem__(self, key, value)
@@ -216,26 +199,90 @@ class ClassTests(unittest.TestCase):
         self.assertTrue(table['two'] == 'my two')
         self.assertTrue(table['three'] == 'my three')
 
-        self.assertTrue(table.Count == 3)                    
+        self.assertTrue(table.Count == 3)
 
+    def testAddAndRemoveClassAttribute(self):
+        
+        from System import TimeSpan
 
+        for i in range(100):
+            TimeSpan.new_method = lambda self: self.TotalMinutes
+            ts = TimeSpan.FromHours(1)
+            self.assertTrue(ts.new_method() == 60)
+            del TimeSpan.new_method
+            self.assertFalse(hasattr(ts, "new_method"))
+
+    def testComparisons(self):
+        from System import DateTimeOffset
+
+        d1 = DateTimeOffset.Parse("2016-11-14")
+        d2 = DateTimeOffset.Parse("2016-11-15")
+
+        self.assertEqual(d1 == d2, False)
+        self.assertEqual(d1 != d2, True)
+
+        self.assertEqual(d1 < d2, True)
+        self.assertEqual(d1 <= d2, True)
+        self.assertEqual(d1 >= d2, False)
+        self.assertEqual(d1 > d2, False)
+
+        self.assertEqual(d1 == d1, True)
+        self.assertEqual(d1 != d1, False)
+
+        self.assertEqual(d1 < d1, False)
+        self.assertEqual(d1 <= d1, True)
+        self.assertEqual(d1 >= d1, True)
+        self.assertEqual(d1 > d1, False)
+
+        self.assertEqual(d2 == d1, False)
+        self.assertEqual(d2 != d1, True)
+
+        self.assertEqual(d2 < d1, False)
+        self.assertEqual(d2 <= d1, False)
+        self.assertEqual(d2 >= d1, True)
+        self.assertEqual(d2 > d1, True)
+
+        self.assertRaises(TypeError, lambda: d1 < None)
+        self.assertRaises(TypeError, lambda: d1 < System.Guid())
+
+        # ClassTest does not implement IComparable
+        c1 = ClassTest()
+        c2 = ClassTest()
+        self.assertRaises(TypeError, lambda: c1 < c2)
+
+    def testSelfCallback(self):
+        """ Test calling back and forth between this and a c# baseclass."""
+        class CallbackUser(Test.SelfCallbackTest):
+            def DoCallback(self):
+                self.PyCallbackWasCalled = False
+                self.SameReference = False
+                return self.Callback(self)
+            def PyCallback(self, self2):
+                self.PyCallbackWasCalled = True
+                self.SameReference = self == self2
+                
+        testobj = CallbackUser()
+        testobj.DoCallback()
+        self.assertTrue(testobj.PyCallbackWasCalled)
+        self.assertTrue(testobj.SameReference)
 
 class ClassicClass:
     def kind(self):
         return 'classic'
 
+
 class NewStyleClass(object):
     def kind(self):
         return 'new-style'
 
 
-
 def test_suite():
     return unittest.makeSuite(ClassTests)
 
+
 def main():
     unittest.TextTestRunner().run(test_suite())
 
+
 if __name__ == '__main__':
     main()
-
