@@ -1,19 +1,10 @@
-# ===========================================================================
-# This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
-# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
-# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE.
-# ===========================================================================
-
 import sys, os, string, unittest, types
 import System
+import six
+
+if six.PY3:
+    unicode = str
 
-# Note: all of these tests are known to fail because Python currently
-# doesn't allow new-style classes to be used as exceptions. I'm leaving
-# the tests in place in to document 'how it ought to work' in the hopes
-# that they'll all pass one day...
 
 class ExceptionTests(unittest.TestCase):
     """Test exception support."""
@@ -21,12 +12,12 @@ class ExceptionTests(unittest.TestCase):
     def testUnifiedExceptionSemantics(self):
         """Test unified exception semantics."""
         from System import Exception, Object
-        import exceptions
-        
-        e = Exception('Something bad happened')
-        self.assertTrue(isinstance(e, exceptions.Exception))
-        self.assertTrue(isinstance(e, Exception))
 
+        e = Exception('Something bad happened')
+        if not six.PY3:
+            import exceptions
+            self.assertTrue(isinstance(e, exceptions.Exception))
+        self.assertTrue(isinstance(e, Exception))
 
     def testStandardExceptionAttributes(self):
         """Test accessing standard exception attributes."""
@@ -43,19 +34,17 @@ class ExceptionTests(unittest.TestCase):
 
         v = e.ToString()
         self.assertTrue(len(v) > 0)
-                       
 
     def testExtendedExceptionAttributes(self):
         """Test accessing extended exception attributes."""
         from Python.Test import ExceptionTest, ExtendedException
         from System import Exception, OverflowException
-        import exceptions
-        
+
         e = ExceptionTest.GetExtendedException()
         self.assertTrue(isinstance(e, ExtendedException))
         self.assertTrue(isinstance(e, OverflowException))
         self.assertTrue(isinstance(e, Exception))
-        
+
         self.assertTrue(e.Message == 'error')
 
         e.Source = 'Test Suite'
@@ -67,10 +56,9 @@ class ExceptionTests(unittest.TestCase):
         self.assertTrue(e.ExtraProperty == 'extra')
         e.ExtraProperty = 'changed'
         self.assertTrue(e.ExtraProperty == 'changed')
-                       
+
         self.assertTrue(e.GetExtraInfo() == 'changed')
-        
-                        
+
     def testRaiseClassException(self):
         """Test class exception propagation."""
         from System import NullReferenceException
@@ -87,13 +75,12 @@ class ExceptionTests(unittest.TestCase):
             self.assertTrue(type is NullReferenceException)
             self.assertTrue(isinstance(value, NullReferenceException))
 
-
     def testRaiseClassExceptionWithValue(self):
         """Test class exception propagation with associated value."""
         from System import NullReferenceException
 
         def test():
-            raise NullReferenceException, 'Aiiieee!'
+            raise NullReferenceException('Aiiieee!')
 
         self.assertRaises(NullReferenceException, test)
 
@@ -105,7 +92,6 @@ class ExceptionTests(unittest.TestCase):
             self.assertTrue(isinstance(value, NullReferenceException))
             self.assertTrue(value.Message == 'Aiiieee!')
 
-
     def testRaiseInstanceException(self):
         """Test instance exception propagation."""
         from System import NullReferenceException
@@ -123,7 +109,6 @@ class ExceptionTests(unittest.TestCase):
             self.assertTrue(isinstance(value, NullReferenceException))
             self.assertTrue(len(value.Message) > 0)
 
-
     def testRaiseInstanceExceptionWithArgs(self):
         """Test instance exception propagation with args."""
         from System import NullReferenceException
@@ -141,7 +126,6 @@ class ExceptionTests(unittest.TestCase):
             self.assertTrue(isinstance(value, NullReferenceException))
             self.assertTrue(value.Message == 'Aiiieee!')
 
-
     def testManagedExceptionPropagation(self):
         """Test propagation of exceptions raised in managed code."""
         from System import Decimal, OverflowException
@@ -151,7 +135,6 @@ class ExceptionTests(unittest.TestCase):
 
         self.assertRaises(OverflowException, test)
 
-
     def testManagedExceptionConversion(self):
         """Test conversion of managed exceptions."""
         from System import Exception, OverflowException
@@ -175,8 +158,7 @@ class ExceptionTests(unittest.TestCase):
         self.assertTrue(v)
 
         v = ExceptionTest.SetWidenedException(OverflowException('error'))
-        self.assertTrue(v)     
-
+        self.assertTrue(v)
 
     def testCatchExceptionFromManagedMethod(self):
         """Test catching an exception from a managed method."""
@@ -185,13 +167,13 @@ class ExceptionTests(unittest.TestCase):
 
         try:
             ExceptionTest().ThrowException()
-        except OverflowException, e:
+        except OverflowException:
+            e = sys.exc_info()[1]
             self.assertTrue(isinstance(e, OverflowException))
             return
 
         raise SystemError('failed to catch exception from managed method')
 
-
     def testCatchExceptionFromManagedProperty(self):
         """Test catching an exception from a managed property."""
         from Python.Test import ExceptionTest
@@ -199,19 +181,20 @@ class ExceptionTests(unittest.TestCase):
 
         try:
             v = ExceptionTest().ThrowProperty
-        except OverflowException, e:
+        except OverflowException:
+            e = sys.exc_info()[1]
             self.assertTrue(isinstance(e, OverflowException))
             return
 
         try:
             ExceptionTest().ThrowProperty = 1
-        except OverflowException, e:
+        except OverflowException:
+            e = sys.exc_info()[1]
             self.assertTrue(isinstance(e, OverflowException))
             return
 
         raise SystemError('failed to catch exception from managed property')
 
-
     def testCatchExceptionManagedClass(self):
         """Test catching the managed class of an exception."""
         from System import OverflowException
@@ -223,11 +206,13 @@ class ExceptionTests(unittest.TestCase):
 
         raise SystemError('failed to catch managed class exception')
 
-
     def testCatchExceptionPythonClass(self):
         """Test catching the python class of an exception."""
         from System import OverflowException
-        from exceptions import Exception
+        if six.PY3:
+            from builtins import Exception
+        else:
+            from exceptions import Exception
 
         try:
             raise OverflowException('overflow')
@@ -236,7 +221,6 @@ class ExceptionTests(unittest.TestCase):
 
         raise SystemError('failed to catch python class exception')
 
-
     def testCatchExceptionBaseClass(self):
         """Test catching the base of an exception."""
         from System import OverflowException, ArithmeticException
@@ -248,7 +232,6 @@ class ExceptionTests(unittest.TestCase):
 
         raise SystemError('failed to catch base exception')
 
-
     def testCatchExceptionNestedBaseClass(self):
         """Test catching the nested base of an exception."""
         from System import OverflowException, SystemException
@@ -260,17 +243,16 @@ class ExceptionTests(unittest.TestCase):
 
         raise SystemError('failed to catch nested base exception')
 
-
     def testCatchExceptionWithAssignment(self):
         """Test catching an exception with assignment."""
         from System import OverflowException
 
         try:
             raise OverflowException('overflow')
-        except OverflowException, e:
+        except OverflowException:
+            e = sys.exc_info()[1]
             self.assertTrue(isinstance(e, OverflowException))
 
-
     def testCatchExceptionUnqualified(self):
         """Test catching an unqualified exception."""
         from System import OverflowException
@@ -282,7 +264,6 @@ class ExceptionTests(unittest.TestCase):
 
         raise SystemError('failed to catch unqualified exception')
 
-
     def testApparentModuleOfException(self):
         """Test the apparent module of an exception."""
         from System import Exception, OverflowException
@@ -290,7 +271,6 @@ class ExceptionTests(unittest.TestCase):
         self.assertTrue(Exception.__module__ == 'System')
         self.assertTrue(OverflowException.__module__ == 'System')
 
-
     def testStrOfException(self):
         """Test the str() representation of an exception."""
         from System import NullReferenceException
@@ -303,26 +283,27 @@ class ExceptionTests(unittest.TestCase):
 
         try:
             Convert.ToDateTime('this will fail')
-        except FormatException, e:
-            msg = unicode(e).encode("utf8") # fix for international installation
-            self.assertTrue(msg.find('System.Convert.ToDateTime') > -1, msg)
+        except FormatException:
+            e = sys.exc_info()[1]
+            msg = unicode(e).encode("utf8")  # fix for international installation
+            self.assertTrue(msg.find(unicode('System.Convert.ToDateTime').encode("utf8")) > -1, msg)
 
-            
     def testPythonCompatOfManagedExceptions(self):
         """Test if managed exceptions are compatible with Python's implementation
         """
         from System import OverflowException
         msg = "A simple message"
-        
+
         e = OverflowException(msg)
-        self.assertEqual(e.message, msg)
-        self.assertTrue(isinstance(e.message, unicode)) # ???
         self.assertEqual(str(e), msg)
         self.assertEqual(unicode(e), msg)
-        
+
         self.assertEqual(e.args, (msg,))
         self.assertTrue(isinstance(e.args, tuple))
-        self.assertEqual(repr(e), "OverflowException('A simple message',)")
+        if six.PY2:
+            self.assertEqual(repr(e), "OverflowException(u'A simple message',)")
+        else:
+            self.assertEqual(repr(e), "OverflowException('A simple message',)")
 
     def testExceptionIsInstanceOfSystemObject(self):
         """Test behavior of isinstance(<managed exception>, System.Object)."""
@@ -336,19 +317,63 @@ class ExceptionTests(unittest.TestCase):
         # without causing a crash in the CPython interpreter). This test is
         # here mainly to remind me to update the caveat in the documentation
         # one day when when exceptions can be new-style classes.
+
+        # This behaviour is now over-shadowed by the implementation of
+        # __instancecheck__ (i.e., overloading isinstance), so for all Python
+        # version >= 2.6 we expect isinstance(<managed exception>, Object) to
+        # be true, even though it does not really subclass Object.
         from System import OverflowException
         from System import Object
-        
+
         o = OverflowException('error')
-        self.assertFalse(isinstance(o, Object))
-    
-        
+
+        if sys.version_info >= (2, 6):
+            self.assertTrue(isinstance(o, Object))
+        else:
+            self.assertFalse(isinstance(o, Object))
+
+    def testPicklingExceptions(self):
+        from System import Exception
+        try:
+            import cPickle as pickle
+        except ImportError:
+            import pickle
+
+        exc = Exception("test")
+        dumped = pickle.dumps(exc)
+        loaded = pickle.loads(dumped)
+
+        self.assertEqual(exc.args, loaded.args)
+
+    def testChainedExceptions(self):
+        if six.PY3:
+            from Python.Test import ExceptionTest
+
+            try:
+                ExceptionTest.ThrowChainedExceptions()
+            except Exception as exc:
+                msgs = [
+                    "Outer exception",
+                    "Inner exception",
+                    "Innermost exception"
+                ]
+
+                for msg in msgs:
+                    self.assertEqual(exc.Message, msg)
+                    self.assertEqual(exc.__cause__, exc.InnerException)
+                    exc = exc.__cause__
+
+            else:
+                self.fail("Test should raise an exception")
+
 
 def test_suite():
     return unittest.makeSuite(ExceptionTests)
 
+
 def main():
     unittest.TextTestRunner().run(test_suite())
 
+
 if __name__ == '__main__':
     main()
